/************************************************************

  This example shows how to create and extend an unlimited
  dataset.  The program first writes integers to a dataset
  with dataspace dimensions of DIM0xDIM1, then closes the
  file.  Next, it reopens the file, reads back the data,
  outputs it to the screen, extends the dataset, and writes
  new data to the extended portions of the dataset.  Finally
  it reopens the file again, reads back the data, and
  outputs it to the screen.

 ************************************************************/

//#include "hdf5.h"
#include "pch.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define FILE            "test.h5"
#define DATASET1        "DS1"
#define DATASET2        "DS2"
#define DIM0            10
//#define DIM1            10
#define EDIM0           100
//#define EDIM1           100
#define CHUNK0          4
//#define CHUNK1          4

 // basic demo code 
extern "C"
{
    __declspec(dllexport) void DisplayHelloFromDLL()
    {
        printf("Hello from DLL !\n");
    }
}

extern "C"
{
    __declspec(dllexport) int SetupFile()
    {
        hid_t           file, space, dset1, dset2, dcpl;    /* Handles */
        herr_t          status;
        hsize_t         dims[1] = { DIM0 },
                        extdims[1] = { EDIM0 },
                        maxdims[1],
                        chunk[1] = { CHUNK0 },
                        start[1],
                        count[1];
        int             wdata[DIM0],          /* Write buffer */
                        wdata2[EDIM0],       /* Write buffer for
                                                       extension */
                        ndims;
        hsize_t         i, j;

        /*
         * Initialize data.
         */
        for (i = 0; i < DIM0; i++)
            wdata[i] = i;

        /*
         * Create a new file using the default properties.
         */
        file = H5Fcreate(FILE, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

        /*
         * Create dataspace with unlimited dimensions.
         */
        maxdims[0] = H5S_UNLIMITED;
        space = H5Screate_simple(2, dims, maxdims);

        /*
         * Create the dataset creation property list, and set the chunk
         * size.
         */
        dcpl = H5Pcreate(H5P_DATASET_CREATE);
        status = H5Pset_chunk(dcpl, 1, chunk);

        clock_t beginning = clock();

        /*
         * Create the unlimited dataset.
         */
        dset1 = H5Dcreate(file, DATASET1, H5T_STD_I32LE, space, H5P_DEFAULT, dcpl,
            H5P_DEFAULT);
        dset2 = H5Dcreate (file, DATASET2, H5T_STD_I32LE, space, H5P_DEFAULT, dcpl,
                    H5P_DEFAULT);

        /*
         * Write the data to the dataset.
         */
        status = H5Dwrite(dset1, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,
            wdata);
        status = H5Dwrite (dset2, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,
                    wdata);

        clock_t end = clock();
        float seconds = (float)(end - beginning);

        printf("time taken to create dataspace & both datasets: %f \n", seconds);

        /*
         * Close and release resources.
         */
        status = H5Pclose(dcpl);
        status = H5Dclose(dset1);
        status = H5Dclose (dset2);
        status = H5Sclose(space);
        status = H5Fclose(file);

        return 0;
    }
}

//extern "C"
//{
//    __declspec(dllexport) int AppendFile(double * data, int idx)
//    {
//        hid_t           file, space, dset1, dset2, dcpl;    /* Handles */
//        herr_t          status;
//        hsize_t         dims[1] = { DIM0 },
//                        extdims[1] = { EDIM0 },
//                        maxdims[1],
//                        chunk[1] = { CHUNK0 },
//                        start[1],
//                        count[1];
//        double          wdata[DIM0],          /* Write buffer */
//                        wdata2[EDIM0],       /* Write buffer for
//                                                        extension */
//                        ndims;
//        hsize_t         i, j;
//
//        /*
//         * Open file and dataset using the default properties.
//         */
//        file = H5Fopen(FILE, H5F_ACC_RDWR, H5P_DEFAULT);
//        dset1 = H5Dopen(file, DATASET1, H5P_DEFAULT);
//        dset2 = H5Dopen(file, DATASET2, H5P_DEFAULT);
//
//        clock_t beginning = clock();
//        /*
//        * Extend the dataset.
//        */
//        status = H5Dset_extent(dset1, extdims);
//        status = H5Dset_extent(dset2, extdims);
//
//        printf("extended dataset, need to write in\n");
//
//        /*
//        * Retrieve the dataspace for the newly extended dataset.
//        */
//        space = H5Dget_space(dset1);
//
//        /*
//        * Initialize data for writing to the extended dataset.
//        */
//        for (i = 0; i < EDIM0; i++)
//            wdata2[i] = i;
//
//        /*
//        * Select the entire dataspace.
//        */
//        status = H5Sselect_all(space);
//
//        /*
//        * Subtract a hyperslab reflecting the original dimensions from the
//        * selection.  The selection now contains only the newly extended
//        * portions of the dataset.
//        */
//        // TODO: instead of hard-coding these numbers, get current dims and base off of them
//        // TODO: getSimpleExtentDims may be helpful for getting current dimension 
// 
//        start[0] = 0; 
//        count[0] = dims[0];
//        // so if i were to want to select a different section to write into for 
//        // the second dataset, i would probably need to open another space 
//        status = H5Sselect_hyperslab(space, H5S_SELECT_NOTB, start, NULL, count,
//            NULL);
//
//        /*
//        * Write the data to the selected portion of the dataset.
//        */
//        status = H5Dwrite(dset1, H5T_NATIVE_DOUBLE, H5S_ALL, space, H5P_DEFAULT,
//            wdata2);
//        status = H5Dwrite(dset2, H5T_NATIVE_DOUBLE, H5S_ALL, space, H5P_DEFAULT,  // not sure if this can be same or also should be sequential 
//            wdata2);
//
//        status = H5Sclose(space); // close dataspace 
//
//        clock_t end = clock();
//        float seconds = (float)(end - beginning) / CLOCKS_PER_SEC;
//        printf("time taken to extend both: %f\n", seconds);
//
//        /*
//        * Close and release resources.
//        */
//        status = H5Dclose(dset1);
//        status = H5Dclose(dset2);
//        status = H5Sclose(space);
//        status = H5Fclose(file);
//
//        printf("wrote extended data"); // not sure if this should be in the for loop or not tbh
//    }
//}
